
'   ==============================================================================
'   Vincenzo Tschinke, Ph.D.
'   Switzerland
'   
'   Contact:
'   
'   newlead|at|me.com
'   ==============================================================================

'   ==============================================================================

'   NLD2MOE

'   Extract information from a NEWLEAD output file (Macromodel)
'   and create a script for MOE.

'   WARNING in the MOE script, the database file given
'   here is over-written if existing.

'   USAGE:
'       cscript /nologo nld2moe.vbs [OPTIONS] INPUT > OUTPUT
'
'   OPTIONS:
'       /h :        This message
'       /d <file> : MOE database file ["nld.mdb"]
'       /n <file> : updated NEWLEAD output file ["nld.mmod"]
'       /l <int>  : length of the atom information line (padded with blanks) [130]

'   Created:

'       Wed Feb  8 15:47:33 CET 2006

'   Revised:

'       March 19, 2009 3:16:46  GMT+01:00
'       April 5, 2009 11:06:12 PM GMT+02:00 (ported to VBScript)

'   ==============================================================================


option explicit

dim usstr

dim MolID
dim RunID

dim help
dim doAll
dim testRun

dim moeMtag
dim moeDBfile
dim nldfile
dim ResNumber
dim LineLength

dim LAST_ITEM 
dim args_index 
dim command_line_args 

dim file_name

usstr = "" & VBCRLF &_
"#   NLD2MOE" & VBCRLF &_
"" & VBCRLF &_
"#   Extract information from a NEWLEAD output file (Macromodel)" & VBCRLF &_
"#   and create a script for MOE." & VBCRLF &_
"" & VBCRLF &_
"#   WARNING in the MOE script, the database file given" & VBCRLF &_
"#   here is over-written if existing." & VBCRLF &_
"" & VBCRLF &_
"" & VBCRLF &_
"#   USAGE:" & VBCRLF &_
"#       cscript /nologo nld2moe.vbs [OPTIONS] INPUT > OUTPUT" & VBCRLF &_
"#" & VBCRLF &_
"#   OPTIONS:" & VBCRLF &_
"#       /h :        This message" & VBCRLF &_
"#       /d <file> : MOE database file" & VBCRLF &_
"#       /n <file> : updated NEWLEAD output file" & VBCRLF &_
"#       /l <int>  : length of the atom information line (padded with blanks)" & VBCRLF &_
"" & VBCRLF &_
"#   Created:" & VBCRLF &_
"" & VBCRLF &_
"#       Wed Feb  8 15:47:33 CET 2006" & VBCRLF &_
"" & VBCRLF &_
"#   Revised:" & VBCRLF &_
"" & VBCRLF &_
"#       March 19, 2009 3:16:46  GMT+01:00" & VBCRLF &_
"#       April 5, 2009 11:06:12 PM GMT+02:00 (ported to VBScript)" & VBCRLF &_
"" & VBCRLF &_
""

MolID = "name"
RunID = "result"

help = false
doAll = false

moeMtag = "mol"
moeDBfile = "nld.mdb"
nldfile = "nld.mmod"
LineLength = 130

const BLANK = " "

const FIRST_ITEM=0 

set command_line_args = wscript.Arguments

file_name = "nld.out"

'Check if any Arguments have been passed to our script. 
if command_line_args.count > 0 then
    LAST_ITEM = command_line_args.count - 1
    for args_index = FIRST_ITEM to LAST_ITEM
        select case (command_line_args(args_index))
            case "/H","/h"
                help = true
            case "/D","/d"
                args_index = args_index + 1
                moeDBfile = command_line_args(args_index)
            case "/N","/n"
                args_index = args_index + 1
                nldfile = command_line_args(args_index)
            case "/L","/l"
                args_index = args_index + 1
                LineLength = command_line_args(args_index)
            case else
                file_name = command_line_args(args_index)
        end select
    next 
end if

if help then
'   Show description and quit
    WScript.Echo usstr
    WScript.Quit
end if

'   Files for the molecular data and the SVL script
dim molOut, svlOut

function file2lines ( file_name )

'   Return an array with all lines in a file
    dim objFSO, objFile
    set objFSO = CreateObject("Scripting.FileSystemObject")
    set objFile = objFSO.OpenTextFile(file_name, 1)
    dim arrFileLines()
    dim i : i = 0
    do until objFile.AtEndOfStream
        redim preserve arrFileLines(i)
        arrFileLines(i) = objFile.ReadLine
        i = i + 1
    loop
    objFile.Close
    file2lines = arrFileLines
end function

function openFile ( file_name )

'   Open a file 
    dim objFSO, objFSOText, objFile
    dim strFile
    strFile = file_name
    
'   Create the File System Object
    set objFSO = CreateObject("Scripting.FileSystemObject")
    set objFile = objFSO.CreateTextFile(strFile)
    set openFile = objFile
end function

function parseLine ( line )

'   Return an array with strings parsed out of a line with blanks as separators
    dim str, sa0
    dim sa1(256)

'   Split using blanks, empty strings and strings with blanks possible  
    str = Trim(line)
    sa0 = split ( str, "  " )
    
    dim idx : idx = 0   
    for each str in sa0
'       Remove blanks   
        str = Trim(str)
'       Remove empty strings
        if Len(str) > 0 then
            sa1(idx) = str
            idx = idx + 1
        end if
    next

'   Return array
    parseLine = sa1
    
end function

'   Read the input in
dim input_lines : input_lines = file2lines ( file_name )

dim strLine, strLen

dim isHeader : isHeader = true
dim count, nat, rnm, molname
dim aStr, strArray
dim res0

dim svlCount : svlCount = 0
dim svlArray()

'   Set up the output files
set molOut = openFile(nldfile)
set svlOut = wscript.stdout

for each strLine in input_lines

    strLine = RTrim ( strLine )

'   This line is a molecule header
    if isHeader then
'       Reset counter       
        count = 0
'       Reset information

'       Get the number of atoms     
        strArray = parseLine(strLine)
        nat =     strArray (0)
        rnm =     strArray (1)
        molname = strArray (2)

'       Reset flag
        isHeader = false
        
'       Process SVL script
        redim preserve svlArray(svlCount)
        svlArray(svlCount) = "ek = db_Write [ '" & moeDBfile & "' , 0 , ['" & RunID & "':""" & rnm & """] ];" & VBCRLF &_ 
                             "db_Write [ '" & moeDBfile & "' , ek , ['" & MolID & "':""" & molname & """] ];" & VBCRLF
        svlCount = svlCount + 1
        
    else 
'       Check if all lines of this group have been read 
        if ( count = Int(nat) ) then
            isHeader = true
        end if  
        
'       Collect data
        res0 = Mid(strLine, 96, 1)
        aStr = Mid(strLine, 1, 94) & res0 & "X" & Mid(strLine, 97)
'       This is a spacer
        if res0 = " " then 
            res0 = "S"
        end if
        strLen = 119 - Len (aStr)
'       Pad with blanks 
        strLine = aStr & String ( strLen , BLANK ) & res0

    end if
    
'   Update counter
    count = count + 1
     
'   Determine final string length : "LineLength" is the optimal length
    strLen = LineLength - Len ( strLine )
'   Pad with blanks 
    strLine = strLine & String ( strLen , BLANK )

    molOut.writeLine strLine    
    
next

molOut.Close

'   Write the SVL script on the standard input
svlOut.writeLine "// Processing file " & file_name & " on " &_
                                    FormatDateTime(Date, 1) &_
                                    VBCRLF & VBCRLF

if svlCount = 0 then
'   No result found : exit
    svlOut.writeLine "Warning 'NEWLEAD: no results found.';" & VBCRLF
    svlOut.writeLine " -1 ;"        ' Error code for MOE
    WScript.Quit
end if

svlOut.writeLine "dbk = db_Open [ '" & moeDBfile & "' , 'create' ];" & VBCRLF
svlOut.writeLine "db_Close [ dbk ];" & VBCRLF & VBCRLF
svlOut.writeLine "db_CreateField ['" & moeDBfile & "', '" & moeMtag & "', 'molecule'];" & VBCRLF
svlOut.writeLine "db_CreateField ['" & moeDBfile & "', '" & RunID & "', 'char'];" & VBCRLF
svlOut.writeLine "db_CreateField ['" & moeDBfile & "', '" & MolID & "', 'char'];" & VBCRLF

'   Write information for each molecule
for each strLine in svlArray
    svlOut.writeLine strLine
next

'   Issue command to read all molecules into MOE DB
svlOut.writeLine "macromodel_ImportMDB ['" & moeDBfile & "', '" & nldfile & "', '" & moeMtag & "', ['overwrite':1]];" & VBCRLF
'   No-error code
svlOut.writeLine " 0 ;"

svlOut.Close

'   END